/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ (https://gitlab.com/DesQ/libdesq)
 * This library contains various  Core and Gui classes which are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#include "DesQIpcServer.hpp"
#include "DesQIpcClient.hpp"

DesQIpcServer::DesQIpcServer( QObject *parent ) : QLocalServer( parent ) {
    connect( this, &QLocalServer::newConnection, this, &DesQIpcServer::handleIncomingConnection );
}


/**
 * Broadcast Message Structure:
 * We use DBus Style signature to decode the signal data.
 * First send the signature
 * Then we send the signal name
 * And finally, we send the signal data
 * signature, signal and data are separated by a two null characters
 **/
void DesQIpcServer::broadcast( QByteArray msg ) {
    // for( DesQIpcClient *client: bcastClients )
    // client->write( msg );
    for ( QLocalSocket *client: bcastClients ) {
        client->write( msg );
        client->flush();
    }
}


void DesQIpcServer::handleIncomingConnection() {
    if ( pending.isEmpty() ) {
        return;
    }

    ////// TODO: Move all this to a thread.

    DesQIpcClient *client = pending.dequeue();
    QString       message( client->readAll() );

    if ( not message.isEmpty() ) {
        if ( message == "register bcast" ) {
            client->write( "ack register" );
            bcastClients << client;
        }

        else if ( message == "register chat" ) {
            client->write( "ack register" );
            xchngClients << client;
        }

        else if ( message == "register msg" ) {
            client->write( "ack register" );

            QByteArray cmd = client->readAll();

            if ( not cmd.isEmpty() ) {
                emit messageReceived( cmd );
                client->write( "ack " + QByteArray::number( cmd.size() ) );
            }

            else {
                qDebug() << serverError() << errorString();
            }

            client->disconnectFromServer();
        }

        else if ( message == "register generic" ) {
            qDebug() << "Generic client registered";
        }

        else {
            qDebug() << "[E]: Bad register request";
            client->write( "bad register" );
            client->disconnectFromServer();
        }
    }

    else {
        qDebug() << "[E]: No response from client. Disconnecting.";
        client->disconnectFromServer();
    }
}


void DesQIpcServer::incomingConnection( quintptr socketDescriptor ) {
    DesQIpcClient *socket = new DesQIpcClient( DesQIpcClient::Generic, this );

    socket->setSocketDescriptor( socketDescriptor );
    pending.enqueue( socket );
    emit newConnection();
}
